# RCAIDE/Library/Methods/Propulsors/Converters/Engine/__init__.py
# (c) Copyright 2023 Aerospace Research Community LLC

""" RCAIDE Package Setup
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from .append_engine_conditions           import append_engine_conditions         
from .compute_throttle_from_power        import compute_throttle_from_power
from .compute_power_from_throttle        import compute_power_from_throttle
from .compute_fuel_flow_rate_from_torque import compute_fuel_flow_rate_from_torque