# RCAIDE/Library/Components/Thermal_Management/Reservoirs/__init__.py

"""
Module containing reservoir components for thermal management systems. This module 
provides classes for fluid storage and management in cooling systems.

See Also
--------
RCAIDE.Library.Components.Thermal_Management.Heat_Exchangers
    Related module for heat exchangers that interface with reservoirs
RCAIDE.Library.Components.Thermal_Management.Accessories
    Related module for pumps and other components that move fluid to/from reservoirs
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ---------------------------------------------------------------------------------------------------------------------- 
from .Reservoir import Reservoir
