# RCAIDE/Framework/Analyses/Weights/__init__.py
# 

"""RCAIDE Package Setup
"""

from .Weights                            import Weights
from .Weights_BWB                        import Weights_BWB
from .Weights_Transport                  import Weights_Transport
from .Weights_UAV                        import Weights_UAV 
from .Weights_General_Aviation           import Weights_General_Aviation
from .Weights_Human_Powered              import Weights_Human_Powered
from .Weights_EVTOL                      import Weights_EVTOL 