# RCAIDE/Library/Components/Nacelles/__init__.py

"""
Module containing nacelle segments for aircraft propulsion system integration.

See Also
--------
RCAIDE.Library.Components.Fuselages
    Related module for fuselage components that may interface with engine nacelles
RCAIDE.Library.Components.Landing_Gear
    Related module for landing gear design which may affect nacelle ground clearance
RCAIDE.Library.Components.Airfoils
    Related module for airfoil definitions that may influence nacelle-wing integration
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------
 
from .Segment                    import Segment 