# RCAIDE/Methods/Energy/Propulsors/Electric_Rotor_Propulsor/__init__.py
# 

""" RCAIDE Package Setup
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from .compute_electric_rotor_performance import compute_electric_rotor_performance
from .append_electric_rotor_conditions   import append_electric_rotor_conditions
from .pack_electric_rotor_residuals      import pack_electric_rotor_residuals
from .unpack_electric_rotor_unknowns     import unpack_electric_rotor_unknowns
from .append_electric_rotor_residual_and_unknown import  append_electric_rotor_residual_and_unknown