# RCAIDE/Library/Attributes/Materials/__init__.py
# 

"""RCAIDE Package Setup
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from .Solid                       import Solid
from .Unidirectional_Carbon_Fiber import Unidirectional_Carbon_Fiber
from .Bidirectional_Carbon_Fiber  import Bidirectional_Carbon_Fiber
from .Carbon_Fiber_Honeycomb      import Carbon_Fiber_Honeycomb
from .Epoxy                       import Epoxy
from .Aluminum_Rib                import Aluminum_Rib
from .Paint                       import Paint
from .Aluminum                    import Aluminum
from .Polyetherimide              import Polyetherimide
from .Acrylic                     import Acrylic
from .Steel                       import Steel
from .Nickel                      import Nickel
from .Titanium                    import Titanium
from .Magnesium                   import Magnesium