
""" RCAIDE Package Setup for rotor wake methods that are directly specified by analyses.
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from RCAIDE.Library.Methods.Propulsors.Converters.Rotor.Wake.Perscribed_Vortex_Wake.compute_fidelity_zero_induced_velocity   import compute_fidelity_zero_induced_velocity 
from RCAIDE.Library.Methods.Propulsors.Converters.Rotor.Wake.Perscribed_Vortex_Wake.compute_wake_contraction_matrix          import compute_wake_contraction_matrix