# RCAIDE/Optimization/Packages/trmm/__init__.py
# 

""" RCAIDE Package Setup
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ---------------------------------------------------------------------------------------------------------------------- 

from RCAIDE.Framework.Optimization.Packages      import trmm 
from RCAIDE.Framework.Optimization.Packages.trmm import TRMM_setup
from RCAIDE.Framework.Optimization.Packages.trmm import Trust_Region
from RCAIDE.Framework.Optimization.Packages.trmm import Trust_Region_Optimization