
""" RCAIDE Package Setup for rotor wake methods that are directly specified by analyses.
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from RCAIDE.Library.Methods.Propulsors.Converters.Rotor.Wake.Perscribed_Vortex_Wake  import compute_fidelity_one_inflow_velocities 
from RCAIDE.Library.Methods.Propulsors.Converters.Rotor.Wake.Perscribed_Vortex_Wake  import compute_wake_induced_velocity