# RCAIDE/Methods/Energy/Propulsors/Modulators/__init__.py
# 

""" RCAIDE Package Setup
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------
from .append_esc_conditions  import  append_esc_conditions
from .compute_esc_performance import compute_voltage_out_from_throttle
from .compute_esc_performance import compute_voltage_in_from_throttle
from .compute_esc_performance import compute_throttle_from_voltages
from .compute_esc_performance import compute_current_in_from_throttle 
