# RCAIDE/Methods/Aerodynamics/Airfoil_Panel_Method/__init__.py
# 

""" RCAIDE Package Setup
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ---------------------------------------------------------------------------------------------------------------------- 

from .aero_coeff             import aero_coeff    
from .airfoil_analysis       import airfoil_analysis 
from .heads_method           import heads_method       
from .hess_smith             import hess_smith               
from .infl_coeff             import infl_coeff       
from .panel_geometry         import panel_geometry   
from .thwaites_method        import thwaites_method    
from .velocity_distribution  import velocity_distribution
from .cf_filter              import cf_filter
from .chordwise_distribution import chordwise_distribution