# 

""" RCAIDE Package Setup
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ---------------------------------------------------------------------------------------------------------------------- 

from .append_battery_conditions               import append_battery_conditions, append_battery_segment_conditions
from .find_ragone_properties                  import find_ragone_properties
from .find_specific_power                     import find_specific_power
from .find_mass_gain_rate                     import find_mass_gain_rate
from .find_total_mass_gain                    import find_total_mass_gain
from .size_module_from_mass                    import size_module_from_mass
from .size_module_from_energy_and_power        import size_module_from_energy_and_power
from .compute_module_properties               import compute_module_properties 