# RCAIDE/Library/Methods/Thermal_Management/Batteries/Liquid_Cooled_Wavy_Channel/__init__.py

""" RCAIDE Package Setup
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ---------------------------------------------------------------------------------------------------------------------- 
 

from .design_wavy_channel                     import design_wavy_channel
from .wavy_channel_geometry_setup             import wavy_channel_geometry_setup
from .wavy_channel_sizing_setup               import wavy_channel_sizing_setup
from .wavy_channel_rating_model               import wavy_channel_rating_model
from .append_wavy_channel_conditions          import append_wavy_channel_conditions
from .append_wavy_channel_conditions          import append_wavy_channel_segment_conditions