# RCAIDE/Library/Methods/Emissions/Emission_Index_Empirical_Method/__init__.py

# Created: 2024, M. Clarke

"""
Emission Index Empirical Method 

This module provides functionality for evaluating emission indices using empirical correlations.

Functions
---------
evaluate_correlation_emissions_indices
    Computes emission indices using empirical correlations 

**Major Assumptions**

* Correlations are valid within the operating range

See Also
--------
RCAIDE.Library.Methods.Emissions.Emission_Index_Empirical_Method.evaluate_correlation_emissions_indices

"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from .evaluate_correlation_emissions_indices  import evaluate_correlation_emissions_indices