# RCAIDE/Methods/Aerodynamics/Fidelity_Zero/__init__.py
# 

""" RCAIDE Package Setup
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ---------------------------------------------------------------------------------------------------------------------- 
from .build_VLM_surrogates                    import build_VLM_surrogates  
from .compute_RHS_matrix                      import compute_RHS_matrix 
from .compute_wing_induced_velocity           import compute_wing_induced_velocity
from .deflect_control_surface                 import deflect_control_surfaces
from .extract_wing_collocation_points         import extract_wing_collocation_points
from .generate_VD_helpers                     import postprocess_VD, compute_panel_area, compute_unit_normal 
from .generate_vortex_distribution            import generate_vortex_distribution
from .make_VLM_wings                          import make_VLM_wings
from .train_VLM_surrogates                    import train_VLM_surrogates
from .VLM                                     import VLM
from .evaluate_VLM                            import *  

