# RCAIDE/Library/Components/Systems/__init__.py 
# 

"""
Module containing aircraft system components for modeling various onboard systems 
and equipment. This module provides base system classes and specific implementations 
for avionics and other aircraft systems.
"""

# ----------------------------------------------------------------------------------------------------------------------
#  IMPORT
# ----------------------------------------------------------------------------------------------------------------------

from .System   import System
from .Avionics import Avionics